`timescale 1ns / 1ps

module beep(clock, left_out, collision, start, point, win, game_running);
   input clock;              // system clock
	input collision;
	input start;
	input point;
	input win;
	input game_running;
   output [19:0] left_out;	// PCM data out to AC'97

	reg [19:0] left_out;
	
	parameter [3:0] STOP = 0,
						 BEEP = 1,
						 START1 = 2,
						 START2 = 3,
						 START3 = 4,
						 START4 = 5,
						 START5 = 6,
						 POINT1 = 7,
						 POINT2 = 8,
						 WIN1 = 9,
						 WIN2 = 10,
						 WIN3 = 11,
						 WIN4 = 12,
						 WIN5 = 13,
						 WIN6 = 14;
						 
	parameter [31:0] wait_time = 20000000;
	parameter [31:0] wait_time1 = 20000000;
	parameter [31:0] wait_time2 = 40000000;
	parameter [31:0] wait_time3 = 60000000;
	parameter [31:0] wait_time4 = 80000000;
	parameter [31:0] wait_time5 = 110000000;
	parameter [31:0] wait_time6 = 140000000;
						 
	reg		[3:0]	 r_cs;
	reg		[3:0]  c_ns;
	
	reg [11:0] counter;
	reg [31:0] time_count;

	
	always @(*)
	begin
		case(r_cs)
		STOP:
		begin
			if (game_running == 1'b0)
				c_ns <= STOP;
			else if(collision == 1'b1)
				c_ns <= BEEP;
			else if (start == 1'b1)
				c_ns <= START1;
			else if (point == 1'b1)
				c_ns <= POINT1;
			else if (win == 1'b1)
				c_ns <= WIN1;
			else
				c_ns <= STOP;
		end
		BEEP:
		begin
			if (time_count == wait_time)
				c_ns <= STOP;
			else
				c_ns <= BEEP;
		end
		START1:
		begin
			if (time_count == wait_time1)
				c_ns <= START2;
			else
				c_ns <= START1;
		end
		START2:
		begin
			if (time_count == wait_time2)
				c_ns <= START3;
			else
				c_ns <= START2;
		end
		START3:
		begin
			if (time_count == wait_time3)
				c_ns <= START4;
			else
				c_ns <= START3;
		end
		START4:
		begin
			if(time_count == wait_time4)
				c_ns <= START5;
			else
				c_ns <= START4;
		end
		START5:
		begin
			if(time_count == wait_time5)
				c_ns <= STOP;
			else
				c_ns <= START5;
		end
		
		POINT1:
		begin
			if(time_count == wait_time1)
				c_ns <= POINT2;
			else
				c_ns <= POINT1;
		end
		POINT2:
		begin
			if(time_count == wait_time2)
				c_ns <= STOP;
			else
				c_ns <= POINT2;
		end
		
		WIN1:
		begin
			if(time_count == wait_time1)
				c_ns <= WIN2;
			else
				c_ns <= WIN1;
		end
		WIN2:
		begin
			if(time_count == wait_time2)
				c_ns <= WIN3;
			else
				c_ns <= WIN2;
		end
		WIN3:
		begin
			if(time_count == wait_time3)
				c_ns <= WIN4;
			else
				c_ns <= WIN3;
		end
		WIN4:
		begin
			if(time_count == wait_time4)
				c_ns <= WIN5;
			else
				c_ns <= WIN4;
		end
		WIN5:
		begin
			if(time_count == wait_time5)
				c_ns <= WIN6;
			else
				c_ns <= WIN5;
		end
		WIN6:
		begin
			if(time_count == wait_time6)
				c_ns <= STOP;
			else
				c_ns <= WIN6;
		end
		
		
		default:
				c_ns <= STOP;
		endcase
	end
	
	always @ (posedge clock) begin
		r_cs <= c_ns;
	end
	
		
	always @ (posedge clock)  begin
		counter <= counter + 1;
		if (r_cs == BEEP)
		begin
			time_count <= time_count + 1;
			left_out[15] <= counter[10];
		end
		
		if (r_cs == START1)
		begin
			time_count <= time_count + 1;
			left_out[15] <= counter[10];
		end
		else if (r_cs == START2)
		begin
			time_count <= time_count + 1;
			left_out[15] <= counter[9];
		end
		else if (r_cs == START3)
		begin
			time_count <= time_count + 1;
			left_out[15] <= counter[8];
		end
		else if (r_cs == START4)
		begin
			time_count <= time_count + 1;
			left_out[15] <= counter[9];
		end
		else if (r_cs == START5)
		begin
			time_count <= time_count + 1;
			left_out[15] <= counter[10];
		end
		
		else if (r_cs == POINT1)
		begin
			time_count <= time_count + 1;
			left_out[16] <= counter[9];
		end
		else if (r_cs == POINT2)
		begin
			time_count <= time_count + 1;
			left_out[16] <= counter[10];
		end
		
		else if (r_cs == WIN1)
		begin
			time_count <= time_count + 1;
			left_out[16] <= counter[10];
		end
		else if (r_cs == WIN2)
		begin
			time_count <= time_count + 1;
			left_out[16] <= counter[9];
		end
		else if (r_cs == WIN3)
		begin
			time_count <= time_count + 1;
			left_out[16] <= counter[8];
		end
		else if (r_cs == WIN4)
		begin
			time_count <= time_count + 1;
			left_out[16] <= counter[7];
		end
		else if (r_cs == WIN5)
		begin
			time_count <= time_count + 1;
			left_out[16] <= counter[8];
		end
		else if (r_cs == WIN6)
		begin
			time_count <= time_count + 1;
			left_out[16] <= counter[7];
		end
		
		else if (r_cs == STOP)
		begin
			time_count <= 0;
			left_out[15] <= 0;
		end
	end
endmodule
