/******************************************************************************
 * bioefilt.h - Bioelectricity filtering
 ******************************************************************************/

#ifndef BIOEFILT_H
#define BIOEFILT_H

/* Filtering functions; all take Q4.11 as input and return Q4.11 as output */

/**
 * A comb filter for Fs=1200Hz that notches at multiples of 60Hz:
 * Bandwidth=0.1Hz, Astop=20dB
 * @param x the input data
 * @param delays a static int[FILTER_60_ORDER]
 */
int filter_60hz(int x, int *delays);

/**
 * An order-2 Butterworth highpass filter for Fs=1200Hz:
 * F3dB=20Hz
 * @param x the input data
 * @param delays a static int[FILTER_EMG_ORDER]
 */
int filter_emg(int x, int *delays);

/* Constants */

#define FILTER_60HZ_ORDER 20
#define FILTER_EMG_ORDER 2
#define FILTER_EMG_NSEC 1

#endif /* BIOEFILT_H */
