#include <fsl.h>
#include "bioefilt.h"

#undef DEBUG_MODE_FSL
#undef DEBUG_MODE_RAW
#undef DEBUG_MODE_SAMPLE
#undef DEBUG_MODE_ACCUM
#undef DEBUG_MODE_CHANNEL
#undef OVERSAMPLING
#define APPLY_FILTER_POWER
#define APPLY_FILTER_HPF

#define NCHANNELS 4
#define DEBUG_CHANNEL 2
#define DECIMATE 4
#define FILTER_PREDEC_ORDER FILTER_PREDEC##DECIMATE##_ORDER
#define filter_predec filter_predec##DECIMATE
#define AVG_LEN 32
#define FSL_ID_ADC 0
#define FSL_ID_CPU 1

const int THRESHOLD_HI[NCHANNELS] = {
    5000,
    5000,
    55000,
    50000,
};

const int THRESHOLD_LO[NCHANNELS] = {
    3000,
    3000,
    5000,
    3000,
};

char char_equiv(unsigned int channel) {
    switch (channel) {
        case 0: return 'z';
        case 1: return 'x';
        case 2: return 'n';
        case 3: return 'm';
        default: return '\0';
    }
}

int main() {
    int pwr_delay[NCHANNELS][FILTER_60HZ_ORDER];
    int emg_delay[NCHANNELS][FILTER_EMG_ORDER];
#ifdef OVERSAMPLING
    int dec_delay[NCHANNELS][FILTER_PREDEC_ORDER];
    int decimate[NCHANNELS];
#endif /* OVERSAMPLING */
    int buf[NCHANNELS][AVG_LEN];
    int pos[NCHANNELS];
    unsigned int accum[NCHANNELS];
    int asserted[NCHANNELS];
    unsigned int channel;
    int sample;
    unsigned int equiv;
    int i;

    for (channel = 0; channel < NCHANNELS; channel++) {
        for (i = 0; i < FILTER_60HZ_ORDER; i++)
            pwr_delay[channel][i] = 0;
        for (i = 0; i < FILTER_EMG_ORDER; i++)
            emg_delay[channel][i] = 0;
#ifdef OVERSAMPLING
        for (i = 0; i < FILTER_PREDEC_ORDER; i++)
            dec_delay[channel][i] = 0;
        decimate[channel] = 0;
#endif /* OVERSAMPLING */
        for (i = 0; i < AVG_LEN; i++)
            buf[channel][i] = 0;
        pos[channel] = 0;
        accum[channel] = 0;
        asserted[channel] = 0;
    }

    while (1) {
#ifdef DEBUG_MODE_FSL
        putfslx(i++, FSL_ID_CPU, FSL_DEFAULT);
#else /* DEBUG_MODE_FSL */
        /* Read data */
        getfslx(sample, FSL_ID_ADC, FSL_DEFAULT);
#ifdef DEBUG_MODE_RAW
        putfslx(sample, FSL_ID_CPU, FSL_NONBLOCKING);
#else /* DEBUG_MODE_RAW */
        /* Unpack data and convert the sample to Q4.11 */
        channel = (sample >> 24) & (NCHANNELS-1);
        sample = (sample & 0x00ffffff) - 0x00000800;
#ifdef OVERSAMPLING
        /* Apply antialiasing filtering */
        sample = filter_predec(sample, dec_delay[channel]);
        /* Check the decimating filter */
        if (++decimate[channel] == DECIMATE) {
            decimate[channel] = 0;
#endif /* OVERSAMPLING */
            /* Apply primary filtering */
#ifdef APPLY_FILTER_POWER
            sample = filter_60hz(sample, pwr_delay[channel]);
#endif /* APPLY_FILTER_POWER */
#ifdef APPLY_FILTER_HPF
            sample = filter_emg(sample, emg_delay[channel]);
#endif /* APPLY_FILTER_HPF */
            /* Take the absolute value */
            if (sample < 0) {
                sample = -sample;
            }
#ifdef DEBUG_MODE_SAMPLE
            if (channel == DEBUG_CHANNEL) {
                putfslx(sample, FSL_ID_CPU, FSL_NONBLOCKING);
            }
#else /* DEBUG_MODE_SAMPLE */
            /* Adjust the moving average */
            accum[channel] -= buf[channel][pos[channel]];
            accum[channel] += (buf[channel][pos[channel]] = sample);
            if (++pos[channel] >= AVG_LEN) {
                pos[channel] = 0;
            }
#ifdef DEBUG_MODE_ACCUM
            if (channel == DEBUG_CHANNEL) {
                putfslx(accum[channel], FSL_ID_CPU, FSL_NONBLOCKING);
            }
#else /* DEBUG_MODE_ACCUM */
            /* Check if the signal should be asserted or reset */
            if ((accum[channel] >= THRESHOLD_HI[channel]) && !asserted[channel]) {
                asserted[channel] = 1;
#ifdef DEBUG_MODE_CHANNEL
                putfslx(channel, FSL_ID_CPU, FSL_NONBLOCKING);
#else /* DEBUG_MODE_CHANNEL */
                equiv = char_equiv(channel);
                putfslx(equiv, FSL_ID_CPU, FSL_NONBLOCKING);
#endif /* DEBUG_MODE_CHANNEL */
            } else if (accum[channel] <= THRESHOLD_LO[channel]) {
                asserted[channel] = 0;
            }
#endif /* DEBUG_MODE_ACCUM */
#endif /* DEBUG_MODE_SAMPLE */
#ifdef OVERSAMPLING
        }
#endif /* OVERSAMPLING */
#endif /* DEBUG_MODE_RAW */
#endif /* DEBUG_MODE_FSL */
    }

    return 0;
}
