#include <xbasic_types.h>
#include <xparameters.h>
#include <xuartlite_l.h>
#include <xgpio.h>
#include <fsl.h>
#include <stdio.h>
#include "constants.h"
#include "shared.h"

#define FSL_ID_CPU 0

char getchar_adc() {
	 int invalid;
    unsigned long data;
    getfslx(data, FSL_ID_CPU, FSL_NONBLOCKING);
    fsl_isinvalid(invalid);
    if (invalid)
        return -1;
	 return (char)data;
}
int getsample() {
    int sample, invalid;
    getfslx(sample, FSL_ID_CPU, FSL_NONBLOCKING);
    fsl_isinvalid(invalid);
    if (invalid)
        return -1;
    return sample;
}
void flushsamples() {
	int sample, invalid;
	do {
		getfslx(sample, FSL_ID_CPU, FSL_NONBLOCKING);
		fsl_isinvalid(invalid);
	} while (!invalid);
}

/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////// MAIN ////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////


int main() 
{
	print("\r\nBeginning Main Game Processor\r\n");
	
	XGpio game_point;
	XGpio game_start_end;
	XGpio game_running;
	
	//Gpios for sending game state information to the audio core
	XGpio_Initialize( &game_point, XPAR_GPIO_POINT_DEVICE_ID );
	XGpio_Initialize( &game_start_end, XPAR_GPIO_GAME_START_END_DEVICE_ID);
	XGpio_Initialize( &game_running, XPAR_GPIO_GAME_RUNNING_DEVICE_ID);
	
	//player 1
	struct Snake * snake1;
	snake1 = buildSnake( snake1, 1 );
	int p1_x_pos = 0;
	int p1_y_pos = 0;
	unsigned int p1_score = 0;
	
	int p1_x_advance = 0;
	int p1_y_advance = 0;
	
	
	//player 2
	struct Snake * snake2;
	snake2 = buildSnake( snake2, 2 );
	int p2_x_pos = 0;
	int p2_y_pos = 0;
	unsigned int p2_score = 0;
	
	int p2_x_advance = 0;
	int p2_y_advance = 0;
	
	int level;
	
	clear_screen();
	
	while(1){
	
		clear_game_area();
		clear_side_area();
	
		XGpio_DiscreteWrite( &game_start_end, 1, 0);
		XGpio_DiscreteWrite( &game_start_end, 2, 0);
		XGpio_DiscreteWrite( &game_running, 1, 1);
		
		//reset player 1
		p1_x_pos = SNAKE1_START_X_POS;
		p1_y_pos = SNAKE1_START_Y_POS;	
		
		int p1_x_advance = SEGMENT_SIZE;
		int p1_y_advance = 0;
		
		p1_score = 0;
		write_score(p1_score,1);
		
		
		//reset player 2
		p2_x_pos = SNAKE2_START_X_POS;
		p2_y_pos = SNAKE2_START_Y_POS;	
		
		int p2_x_advance = -SEGMENT_SIZE;
		int p2_y_advance = 0;
		
		p2_score = 0;
		write_score(p2_score,2);
		
		//set game up
		int gameover = 0;
		
		decorate_screen();
		
		level = -1;
		level = add_obstacles(level);
		
		// setup players on screen
		reset_snake( snake1 );
		draw_snake( snake1 );
		
		reset_snake( snake2 );
		draw_snake( snake2 );
		
		
		fflush(stdin);
		
		start_label( 0xFFFFFFFF );
		
		print("Press Q\r\n");
		
		//pressing 'q' starts the game (snakes start moving)
		while( getchar() != 'q' ){
		
		}
		print("Game Start\r\n");
		XGpio_DiscreteWrite( &game_start_end, CHANNEL_START_GAME, 1);
		XGpio_DiscreteWrite( &game_start_end, CHANNEL_START_GAME, 0);
				
		start_label( BACKGROUND_COLOUR );
		
		add_obstacles(level);
		
		add_random_stuff();

		flushsamples();
		while(!gameover){
			
         char input = getchar_adc();
         //char input = getchar();
         //char input = 't';
				
				//int sample = getsample();
				//xil_printf("%3x\r\n", sample);
				//if (sample != -1) xil_printf("%d\r\n", sample);

			//p1: turn left
			if(input == 'z'){
                print("P1L\r\n");
				if( p1_x_advance > 0 ){
					p1_x_advance = 0;
					p1_y_advance = -SEGMENT_SIZE;
				}
				else if( p1_x_advance < 0 ){
					p1_x_advance = 0;
					p1_y_advance = SEGMENT_SIZE;
				}
				else if( p1_y_advance > 0 ){
					p1_x_advance = SEGMENT_SIZE;
					p1_y_advance = 0;
				}
				else if( p1_y_advance < 0 ){
					p1_x_advance = -SEGMENT_SIZE;
					p1_y_advance = 0;
				}
			}
			//p1: turn right
			else if(input == 'x'){
                print("P1R\r\n");
				if( p1_x_advance > 0 ){
					p1_x_advance = 0;
					p1_y_advance = SEGMENT_SIZE;
				}
				else if( p1_x_advance < 0 ){
					p1_x_advance = 0;
					p1_y_advance = -SEGMENT_SIZE;
				}
				else if( p1_y_advance > 0 ){
					p1_x_advance = -SEGMENT_SIZE;
					p1_y_advance = 0;
				}
				else if( p1_y_advance < 0 ){
					p1_x_advance = SEGMENT_SIZE;
					p1_y_advance = 0;
				}
			}
		
			//p2: turn left
			if(input == 'n'){
                print("P2L\r\n");
				if( p2_x_advance > 0 ){
					p2_x_advance = 0;
					p2_y_advance = -SEGMENT_SIZE;
				}
				else if( p2_x_advance < 0 ){
					p2_x_advance = 0;
					p2_y_advance = SEGMENT_SIZE;
				}
				else if( p2_y_advance > 0 ){
					p2_x_advance = SEGMENT_SIZE;
					p2_y_advance = 0;
				}
				else if( p2_y_advance < 0 ){
					p2_x_advance = -SEGMENT_SIZE;
					p2_y_advance = 0;
				}
			}
			//p2: turn right
			else if(input == 'm'){
                print("P2R\r\n");
				if( p2_x_advance > 0 ){
					p2_x_advance = 0;
					p2_y_advance = SEGMENT_SIZE;
				}
				else if( p2_x_advance < 0 ){
					p2_x_advance = 0;
					p2_y_advance = -SEGMENT_SIZE;
				}
				else if( p2_y_advance > 0 ){
					p2_x_advance = -SEGMENT_SIZE;
					p2_y_advance = 0;
				}
				else if( p2_y_advance < 0 ){
					p2_x_advance = SEGMENT_SIZE;
					p2_y_advance = 0;
				}
			}

			
			//advance players
			p1_x_pos += p1_x_advance;
			p1_y_pos += p1_y_advance;
			
			p2_x_pos += p2_x_advance;
			p2_y_pos += p2_y_advance;
			
			
			//check if they hit the top or bottom of the screen
			if( p1_y_pos < 0 || p1_y_pos > 480 ){
				//player 1 died
				gameover = 1;
				add_skull(1, 0xFFFFFFFF);
				XGpio_DiscreteWrite( &game_point, 1, 1);
				XGpio_DiscreteWrite( &game_point, 1, 0);
			}
			if( p2_y_pos < 0 || p2_y_pos > 480 ){
				//player 2 died
				gameover += 2;
				add_skull(2, 0xFFFFFFFF);
				XGpio_DiscreteWrite( &game_point, 1, 1);
				XGpio_DiscreteWrite( &game_point, 1, 0);
			}
			
			unsigned int p1_pixel;
			unsigned int p2_pixel;
			
			int p1_x = p1_x_pos;
			int p2_x = p2_x_pos;
			
			//check each pixel of new head segment for 'events'
			while(  (p1_x < (p1_x_pos+SEGMENT_SIZE)) && (p2_x < (p2_x_pos+SEGMENT_SIZE)) && (gameover == 0) ){
				int p1_y = p1_y_pos;
				int p2_y = p2_y_pos;
				while(  (p1_y < (p1_y_pos+SEGMENT_SIZE)) && (p2_y < (p2_y_pos+SEGMENT_SIZE)) && (gameover == 0) ){
			
					p1_pixel = read_pixel(p1_x,p1_y);
					p2_pixel = read_pixel(p2_x,p2_y);
					
					//if the pixel is black or any snake colour then its gameover
					if( p1_pixel == 0 || p1_pixel == SNAKE1_COLOUR || p1_pixel == SNAKE2_COLOUR ){
						gameover = 1;
						add_skull(1, 0xFFFFFFFF);
						XGpio_DiscreteWrite( &game_point, 1, 1);
						XGpio_DiscreteWrite( &game_point, 1, 0);
					}
					if( p2_pixel == 0 || p2_pixel == SNAKE1_COLOUR || p2_pixel == SNAKE2_COLOUR ){
						gameover += 2;
						add_skull(2, 0xFFFFFFFF);
						XGpio_DiscreteWrite( &game_point, 1, 1);
						XGpio_DiscreteWrite( &game_point, 1, 0);
					}
					//if the pixel is gem coloured...
					if( p1_pixel == GEM_COLOUR ){
						
						clear_gem(p1_x,p1_y);
						
						grow_snake( snake1 );
						p1_score++;
						write_score(p1_score,1);
						
						add_gem();
						
						if( p1_score == 10 ){
							gameover = 2;
						}
						else{
							XGpio_DiscreteWrite( &game_point, 2, 1);
							XGpio_DiscreteWrite( &game_point, 2, 0);
						}
						
					}
					//if the pixel is gem coloured...
					if( p2_pixel == GEM_COLOUR ){
						
						clear_gem(p2_x,p2_y);
						
						grow_snake( snake2 );
						p2_score++;
						write_score(p2_score,2);
						
						add_gem();
						
						if( p2_score == 10 ){
							gameover += 1;
						}
						else{
							XGpio_DiscreteWrite( &game_point, 2, 1);
							XGpio_DiscreteWrite( &game_point, 2, 0);
						}
					}
					p1_y++;
					p2_y++;
				}
				p1_x++;
				p2_x++;
			}
			
			if(gameover == 0){
				update_snake( snake1, p1_x_pos, p1_y_pos );
				update_snake( snake2, p2_x_pos, p2_y_pos );
			}
			else{
			
				XGpio_DiscreteWrite( &game_start_end, CHANNEL_END_GAME, 1);
				XGpio_DiscreteWrite( &game_start_end, CHANNEL_END_GAME, 0);
				
				clear_game_area();
				//if gameover == 1 -> player 1 lost
				if( gameover == 1 ){
					P(244,50,0xFFFFFFFF);
					two(266,50,0xFFFFFFFF);
					
					W(310,50,0xFFFFFFFF);
					I(332,50,0xFFFFFFFF);
					N(354,50,0xFFFFFFFF);
					five(376,50,0xFFFFFFFF);
				}
				//if gameover == 2 -> player 2 lost
				if( gameover == 2 ){
					P(244,50,0xFFFFFFFF);
					one(266,50,0xFFFFFFFF);
					
					W(310,50,0xFFFFFFFF);
					I(332,50,0xFFFFFFFF);
					N(354,50,0xFFFFFFFF);
					five(376,50,0xFFFFFFFF);
				}
				//if gameover == 3 -> tie
				if( gameover == 3 ){
					T(288,50,0xFFFFFFFF);
					I(310,50,0xFFFFFFFF);
					E(332,50,0xFFFFFFFF);
				}
				int l;
				for( l=0; l<25000000; l++ ){
					
				}
				
				write_score(0,1);
				write_score(0,2);
				
			}
			
			//wait time
			int k;
			for( k=0; k<300000; k++ ){
			
			}
		}
	}
}



