#include <stdlib.h>
#include "shared.h"
#include "constants.h"

/////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////// DRAWING /////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////

//Writes the given pixel value to the given screen at the given x-y location
void write_pixel(unsigned int value, unsigned int x, unsigned int y)
{
	( (unsigned int *) VIDEO_OUT_BASEADDR)[(y * 1024) + x] = value;
}

unsigned int read_pixel(unsigned int x, unsigned int y)
{
	return ( (unsigned int *) VIDEO_OUT_BASEADDR)[(y * 1024) + x];
}

//writes entire snake segment to video memory
void write_snake_segment ( unsigned int colour, int x_pos, int y_pos ){
	
	int i;
	int j;
	
	for( i=x_pos; i<(x_pos+SEGMENT_SIZE); i++ ){
		for( j=y_pos; j<(y_pos+SEGMENT_SIZE); j++ ){
			write_pixel(colour, i, j);
		}
	}
}


//blackness
void clear_screen()
{
	int x;
	int y;
	for ( y = 0; y < 480; y++)
	{
		for ( x = 0; x < 640; x++)
		{
			write_pixel(0, x, y);
		}
	}
}

//flashy screen reset
void decorate_screen() {
	int x;
	int y;
	int i;
	
	
	for (i = 0; i < 5; i++){
		for ( y = 0; y < 480; y++)
		{
			for ( x = 80; x < 560; x++)
			{
				if( i < 4 ){
					write_pixel((i*0xF5A39EF7), x, y);
				}
				else{
					write_pixel(BACKGROUND_COLOUR, x, y);
				}
			}
		}
	}
}

//clears only the "playable surface"
void clear_game_area(){
	int x;
	int y;	
	
	for ( y = 0; y < 480; y++)
	{
		for ( x = 80; x < 560; x++)
		{
			write_pixel(BACKGROUND_COLOUR, x, y);
		}
	}
}

//clears only the "side area"
void clear_side_area(){
	int x;
	int y;
	
	for ( y = 76; y < 480; y++)
	{
		for ( x = 0; x < 80; x++)
		{
			write_pixel(0, x, y);
			write_pixel(0, x+560, y);
		}
	}
}


//picks 1 out of 4 stages to play on
int add_obstacles( int q ){

	int i;
	int level;
	if( q == -1 ){
		level = rand() % 4;
	}
	else{
		level = q;
	}
	

	if( level == 0 ){
		for( i=0; i<100; i+=4 ){
			write_snake_segment(0,410+i,150);
			write_snake_segment(0,410+i,146);
			
			write_snake_segment(0,410,150-i);
			write_snake_segment(0,414,150-i);
			
			
			write_snake_segment(0,230-i,330);
			write_snake_segment(0,230-i,334);

			write_snake_segment(0,230,330+i);
			write_snake_segment(0,226,330+i);
		}
	}
	else if ( level == 1 ){
		for( i=0; i<100; i+=4 ){
			write_snake_segment(0,316+i,236);
			write_snake_segment(0,316+i,240);
			
			write_snake_segment(0,316,236-i);
			write_snake_segment(0,320,236-i);
			
			
			write_snake_segment(0,316-i,236);
			write_snake_segment(0,316-i,240);

			write_snake_segment(0,316,236+i);
			write_snake_segment(0,320,236+i);
		}
	}
	else if ( level == 2 ){
		for( i=0; i<100; i+=4 ){
			write_snake_segment(0,460+i,236);
			write_snake_segment(0,460+i,240);
			
			write_snake_segment(0,316,480-i);
			write_snake_segment(0,320,480-i);
			
			
			write_snake_segment(0,176-i,236);
			write_snake_segment(0,176-i,240);

			write_snake_segment(0,316,0+i);
			write_snake_segment(0,320,0+i);
		}
	}
	return level;
}

//displays skull when player collides with the edge of the screen itself or the other player
void add_skull( int player, unsigned int colour ){

	int x;
	int y;

	if( player == 1 ){
		x = 20;
		y = 76;
	}
	else if( player == 2 ){
		x = 580;
		y = 76;
	}

	write_snake_segment(colour,x+8,y);
	write_snake_segment(colour,x+12,y);
	write_snake_segment(colour,x+16,y);
	write_snake_segment(colour,x+20,y);
	write_snake_segment(colour,x+24,y);
	write_snake_segment(colour,x+28,y);
	
	
	write_snake_segment(colour,x+4,y+4);
	write_snake_segment(colour,x+8,y+4);
	write_snake_segment(colour,x+12,y+4);
	write_snake_segment(colour,x+16,y+4);
	write_snake_segment(colour,x+20,y+4);
	write_snake_segment(colour,x+24,y+4);
	write_snake_segment(colour,x+28,y+4);
	write_snake_segment(colour,x+32,y+4);
	
	
	write_snake_segment(colour,x,y+8);
	write_snake_segment(colour,x+4,y+8);
	write_snake_segment(colour,x+12,y+8);
	write_snake_segment(colour,x+16,y+8);
	write_snake_segment(colour,x+20,y+8);
	write_snake_segment(colour,x+24,y+8);
	write_snake_segment(colour,x+32,y+8);
	write_snake_segment(colour,x+36,y+8);
	
	
	write_snake_segment(colour,x,y+12);
	write_snake_segment(colour,x+4,y+12);
	write_snake_segment(colour,x+16,y+12);
	write_snake_segment(colour,x+20,y+12);
	write_snake_segment(colour,x+32,y+12);
	write_snake_segment(colour,x+36,y+12);
	
	
	write_snake_segment(colour,x,y+16);
	write_snake_segment(colour,x+4,y+16);
	write_snake_segment(colour,x+8,y+16);
	write_snake_segment(colour,x+16,y+16);
	write_snake_segment(colour,x+20,y+16);
	write_snake_segment(colour,x+28,y+16);
	write_snake_segment(colour,x+32,y+16);
	write_snake_segment(colour,x+36,y+16);
	
	
	write_snake_segment(colour,x,y+20);
	write_snake_segment(colour,x+4,y+20);
	write_snake_segment(colour,x+8,y+20);
	write_snake_segment(colour,x+12,y+20);
	write_snake_segment(colour,x+16,y+20);
	write_snake_segment(colour,x+20,y+20);
	write_snake_segment(colour,x+24,y+20);
	write_snake_segment(colour,x+28,y+20);
	write_snake_segment(colour,x+32,y+20);
	write_snake_segment(colour,x+36,y+20);
	
	
	write_snake_segment(colour,x+4,y+24);
	write_snake_segment(colour,x+8,y+24);
	write_snake_segment(colour,x+12,y+24);
	write_snake_segment(colour,x+24,y+24);
	write_snake_segment(colour,x+28,y+24);
	write_snake_segment(colour,x+32,y+24);

	
	write_snake_segment(colour,x+12,y+28);
	write_snake_segment(colour,x+16,y+28);
	write_snake_segment(colour,x+20,y+28);
	write_snake_segment(colour,x+24,y+28);

	
	write_snake_segment(colour,x+12,y+32);
	write_snake_segment(colour,x+24,y+32);

}

//adds gems on the sides of the screen
void add_big_gem(int x, int y){

	write_snake_segment(GEM_COLOUR,x+16,y);
	write_snake_segment(GEM_COLOUR,x+20,y);
	write_snake_segment(GEM_COLOUR,x+24,y);

	write_snake_segment(GEM_COLOUR,x+8,y+4);
	write_snake_segment(GEM_COLOUR,x+12,y+4);
	write_snake_segment(GEM_COLOUR,x+16,y+4);
	write_snake_segment(GEM_COLOUR,x+20,y+4);
	write_snake_segment(GEM_COLOUR,x+24,y+4);
	write_snake_segment(GEM_COLOUR,x+28,y+4);
	write_snake_segment(GEM_COLOUR,x+32,y+4);
	
	write_snake_segment(GEM_COLOUR,x,y+8);
	write_snake_segment(GEM_COLOUR,x+4,y+8);
	write_snake_segment(GEM_COLOUR,x+8,y+8);
	write_snake_segment(GEM_COLOUR,x+12,y+8);
	write_snake_segment(GEM_COLOUR,x+16,y+8);
	write_snake_segment(GEM_COLOUR,x+20,y+8);
	write_snake_segment(GEM_COLOUR,x+24,y+8);
	write_snake_segment(GEM_COLOUR,x+28,y+8);
	write_snake_segment(GEM_COLOUR,x+32,y+8);
	write_snake_segment(GEM_COLOUR,x+36,y+8);
	write_snake_segment(GEM_COLOUR,x+40,y+8);
	
	write_snake_segment(GEM_COLOUR,x+4,y+12);
	write_snake_segment(GEM_COLOUR,x+8,y+12);
	write_snake_segment(GEM_COLOUR,x+12,y+12);
	write_snake_segment(GEM_COLOUR,x+16,y+12);
	write_snake_segment(GEM_COLOUR,x+20,y+12);
	write_snake_segment(GEM_COLOUR,x+24,y+12);
	write_snake_segment(GEM_COLOUR,x+28,y+12);
	write_snake_segment(GEM_COLOUR,x+32,y+12);
	write_snake_segment(GEM_COLOUR,x+36,y+12);
	
	write_snake_segment(GEM_COLOUR,x+8,y+16);
	write_snake_segment(GEM_COLOUR,x+12,y+16);
	write_snake_segment(GEM_COLOUR,x+16,y+16);
	write_snake_segment(GEM_COLOUR,x+20,y+16);
	write_snake_segment(GEM_COLOUR,x+24,y+16);
	write_snake_segment(GEM_COLOUR,x+28,y+16);
	write_snake_segment(GEM_COLOUR,x+32,y+16);
	                           
	write_snake_segment(GEM_COLOUR,x+12,y+20);
	write_snake_segment(GEM_COLOUR,x+16,y+20);
	write_snake_segment(GEM_COLOUR,x+20,y+20);
	write_snake_segment(GEM_COLOUR,x+24,y+20);
	write_snake_segment(GEM_COLOUR,x+28,y+20);
	                           
	write_snake_segment(GEM_COLOUR,x+16,y+24);
	write_snake_segment(GEM_COLOUR,x+20,y+24);
	write_snake_segment(GEM_COLOUR,x+24,y+24);
	                           
	write_snake_segment(GEM_COLOUR,x+20,y+28);
}


//adds gem at a random location
void add_gem(){

	int i;
	int j;
	int x;
	int y;
	int occupied;

	do{
		occupied = 0;
		x = rand() % 440 + 90;
		y = rand() % 440 + 10;
	
		for( i=x-12; ((i<(x+36)) && (occupied != 1)); i++ ){
			for( j=y-12; ((j<(y+36)) && (occupied != 1)); j++ ){
				unsigned int pixel = read_pixel(i,j);
				if( (pixel == GEM_COLOUR) || (pixel == 0) || (pixel == SNAKE1_COLOUR) || (pixel == SNAKE2_COLOUR) ){
					occupied = 1;
				}
			}
		}
	}while( occupied == 1 );

	write_pixel(GEM_COLOUR,x+4,y);
	write_pixel(GEM_COLOUR,x+5,y);
	write_pixel(GEM_COLOUR,x+6,y);

	write_pixel(GEM_COLOUR,x+2,y+1);
	write_pixel(GEM_COLOUR,x+3,y+1);
	write_pixel(GEM_COLOUR,x+4,y+1);
	write_pixel(GEM_COLOUR,x+5,y+1);
	write_pixel(GEM_COLOUR,x+6,y+1);
	write_pixel(GEM_COLOUR,x+7,y+1);
	write_pixel(GEM_COLOUR,x+8,y+1);
	
	write_pixel(GEM_COLOUR,x,y+2);
	write_pixel(GEM_COLOUR,x+1,y+2);
	write_pixel(GEM_COLOUR,x+2,y+2);
	write_pixel(GEM_COLOUR,x+3,y+2);
	write_pixel(GEM_COLOUR,x+4,y+2);
	write_pixel(GEM_COLOUR,x+5,y+2);
	write_pixel(GEM_COLOUR,x+6,y+2);
	write_pixel(GEM_COLOUR,x+7,y+2);
	write_pixel(GEM_COLOUR,x+8,y+2);
	write_pixel(GEM_COLOUR,x+9,y+2);
	write_pixel(GEM_COLOUR,x+10,y+2);
	
	write_pixel(GEM_COLOUR,x+1,y+3);
	write_pixel(GEM_COLOUR,x+2,y+3);
	write_pixel(GEM_COLOUR,x+3,y+3);
	write_pixel(GEM_COLOUR,x+4,y+3);
	write_pixel(GEM_COLOUR,x+5,y+3);
	write_pixel(GEM_COLOUR,x+6,y+3);
	write_pixel(GEM_COLOUR,x+7,y+3);
	write_pixel(GEM_COLOUR,x+8,y+3);
	write_pixel(GEM_COLOUR,x+9,y+3);
	
	write_pixel(GEM_COLOUR,x+2,y+4);
	write_pixel(GEM_COLOUR,x+3,y+4);
	write_pixel(GEM_COLOUR,x+4,y+4);
	write_pixel(GEM_COLOUR,x+5,y+4);
	write_pixel(GEM_COLOUR,x+6,y+4);
	write_pixel(GEM_COLOUR,x+7,y+4);
	write_pixel(GEM_COLOUR,x+8,y+4);
	                           
	write_pixel(GEM_COLOUR,x+3,y+5);
	write_pixel(GEM_COLOUR,x+4,y+5);
	write_pixel(GEM_COLOUR,x+5,y+5);
	write_pixel(GEM_COLOUR,x+6,y+5);
	write_pixel(GEM_COLOUR,x+7,y+5);
	                           
	write_pixel(GEM_COLOUR,x+4,y+6);
	write_pixel(GEM_COLOUR,x+5,y+6);
	write_pixel(GEM_COLOUR,x+6,y+6);
	                           
	write_pixel(GEM_COLOUR,x+5,y+7);
}

//adds "gems"
void add_random_stuff()
{
	int k;

	for( k=0; k<15; k++ ){
		add_gem();
	}
}

//clears gem by erasing all gem coloured pixels within a box large enough
//to account for the snake hitting it at any pixel
void clear_gem( int x, int y ){

	int i;
	int j;

	for( i = x-12; i<x+24; i++ )
	{
		for( j = y-12; j<y+24; j++ ){
		
			if( read_pixel(i,j) == GEM_COLOUR ){
				write_pixel((BACKGROUND_COLOUR),i,j);
			}
		}
	}
}


void zero(unsigned int x, unsigned int y, unsigned int colour)
{
	write_snake_segment(colour,x,y);
	write_snake_segment(colour,x+4,y);
	write_snake_segment(colour,x+8,y);
	write_snake_segment(colour,x+12,y);
	write_snake_segment(colour,x+16,y);
	
	write_snake_segment(colour,x,y+4);
	write_snake_segment(colour,x+16,y+4);
	
	write_snake_segment(colour,x,y+8);
	write_snake_segment(colour,x+16,y+8);
	
	write_snake_segment(colour,x,y+12);
	write_snake_segment(colour,x+16,y+12);
	
	write_snake_segment(colour,x,y+16);
	write_snake_segment(colour,x+4,y+16);
	write_snake_segment(colour,x+8,y+16);
	write_snake_segment(colour,x+12,y+16);
	write_snake_segment(colour,x+16,y+16);
}

void one(unsigned int x, unsigned int y, unsigned int colour)
{
	write_snake_segment(colour,x+8,y);
	
	write_snake_segment(colour,x+8,y+4);
	
	write_snake_segment(colour,x+8,y+8);
	
	write_snake_segment(colour,x+8,y+12);
	
	write_snake_segment(colour,x+8,y+16);
}

void two(unsigned int x, unsigned int y, unsigned int colour)
{
	write_snake_segment(colour,x,y);
	write_snake_segment(colour,x+4,y);
	write_snake_segment(colour,x+8,y);
	write_snake_segment(colour,x+12,y);
	write_snake_segment(colour,x+16,y);
	
	write_snake_segment(colour,x+16,y+4);
	
	write_snake_segment(colour,x,y+8);
	write_snake_segment(colour,x+4,y+8);
	write_snake_segment(colour,x+8,y+8);
	write_snake_segment(colour,x+12,y+8);
	write_snake_segment(colour,x+16,y+8);
	
	write_snake_segment(colour,x,y+12);
	
	write_snake_segment(colour,x,y+16);
	write_snake_segment(colour,x+4,y+16);
	write_snake_segment(colour,x+8,y+16);
	write_snake_segment(colour,x+12,y+16);
	write_snake_segment(colour,x+16,y+16);
}

void three(unsigned int x, unsigned int y, unsigned int colour)
{
	write_snake_segment(colour,x,y);
	write_snake_segment(colour,x+4,y);
	write_snake_segment(colour,x+8,y);
	write_snake_segment(colour,x+12,y);
	write_snake_segment(colour,x+16,y);
	
	write_snake_segment(colour,x+16,y+4);
	
	write_snake_segment(colour,x,y+8);
	write_snake_segment(colour,x+4,y+8);
	write_snake_segment(colour,x+8,y+8);
	write_snake_segment(colour,x+12,y+8);
	write_snake_segment(colour,x+16,y+8);
	
	write_snake_segment(colour,x+16,y+12);
	
	write_snake_segment(colour,x,y+16);
	write_snake_segment(colour,x+4,y+16);
	write_snake_segment(colour,x+8,y+16);
	write_snake_segment(colour,x+12,y+16);
	write_snake_segment(colour,x+16,y+16);
}

void four(unsigned int x, unsigned int y, unsigned int colour)
{
	write_snake_segment(colour,x,y);
	write_snake_segment(colour,x+16,y);
	
	write_snake_segment(colour,x,y+4);
	write_snake_segment(colour,x+16,y+4);
	
	write_snake_segment(colour,x,y+8);
	write_snake_segment(colour,x+4,y+8);
	write_snake_segment(colour,x+8,y+8);
	write_snake_segment(colour,x+12,y+8);
	write_snake_segment(colour,x+16,y+8);
	
	write_snake_segment(colour,x+16,y+12);
	
	write_snake_segment(colour,x+16,y+16);
}

void five(unsigned int x, unsigned int y, unsigned int colour)
{
	write_snake_segment(colour,x,y);
	write_snake_segment(colour,x+4,y);
	write_snake_segment(colour,x+8,y);
	write_snake_segment(colour,x+12,y);
	write_snake_segment(colour,x+16,y);
	
	write_snake_segment(colour,x,y+4);
	
	write_snake_segment(colour,x,y+8);
	write_snake_segment(colour,x+4,y+8);
	write_snake_segment(colour,x+8,y+8);
	write_snake_segment(colour,x+12,y+8);
	write_snake_segment(colour,x+16,y+8);
	
	write_snake_segment(colour,x+16,y+12);
	
	write_snake_segment(colour,x,y+16);
	write_snake_segment(colour,x+4,y+16);
	write_snake_segment(colour,x+8,y+16);
	write_snake_segment(colour,x+12,y+16);
	write_snake_segment(colour,x+16,y+16);
}

void six(unsigned int x, unsigned int y, unsigned int colour)
{
	write_snake_segment(colour,x,y);
	write_snake_segment(colour,x+4,y);
	write_snake_segment(colour,x+8,y);
	write_snake_segment(colour,x+12,y);
	write_snake_segment(colour,x+16,y);
	
	write_snake_segment(colour,x,y+4);
	
	write_snake_segment(colour,x,y+8);
	write_snake_segment(colour,x+4,y+8);
	write_snake_segment(colour,x+8,y+8);
	write_snake_segment(colour,x+12,y+8);
	write_snake_segment(colour,x+16,y+8);
	
	write_snake_segment(colour,x,y+12);
	write_snake_segment(colour,x+16,y+12);
	
	write_snake_segment(colour,x,y+16);
	write_snake_segment(colour,x+4,y+16);
	write_snake_segment(colour,x+8,y+16);
	write_snake_segment(colour,x+12,y+16);
	write_snake_segment(colour,x+16,y+16);
}

void seven(unsigned int x, unsigned int y, unsigned int colour)
{
	write_snake_segment(colour,x,y);
	write_snake_segment(colour,x+4,y);
	write_snake_segment(colour,x+8,y);
	write_snake_segment(colour,x+12,y);
	write_snake_segment(colour,x+16,y);
	
	write_snake_segment(colour,x+16,y+4);
	
	write_snake_segment(colour,x+16,y+8);
	
	write_snake_segment(colour,x+16,y+12);
	
	write_snake_segment(colour,x+16,y+16);
}

void eight(unsigned int x, unsigned int y, unsigned int colour)
{
	write_snake_segment(colour,x,y);
	write_snake_segment(colour,x+4,y);
	write_snake_segment(colour,x+8,y);
	write_snake_segment(colour,x+12,y);
	write_snake_segment(colour,x+16,y);
	
	write_snake_segment(colour,x,y+4);
	write_snake_segment(colour,x+16,y+4);
	
	write_snake_segment(colour,x,y+8);
	write_snake_segment(colour,x+4,y+8);
	write_snake_segment(colour,x+8,y+8);
	write_snake_segment(colour,x+12,y+8);
	write_snake_segment(colour,x+16,y+8);
	
	write_snake_segment(colour,x,y+12);
	write_snake_segment(colour,x+16,y+12);
	
	write_snake_segment(colour,x,y+16);
	write_snake_segment(colour,x+4,y+16);
	write_snake_segment(colour,x+8,y+16);
	write_snake_segment(colour,x+12,y+16);
	write_snake_segment(colour,x+16,y+16);
}

void nine(unsigned int x, unsigned int y, unsigned int colour)
{
	write_snake_segment(colour,x,y);
	write_snake_segment(colour,x+4,y);
	write_snake_segment(colour,x+8,y);
	write_snake_segment(colour,x+12,y);
	write_snake_segment(colour,x+16,y);
	
	write_snake_segment(colour,x,y+4);
	write_snake_segment(colour,x+16,y+4);
	
	write_snake_segment(colour,x,y+8);
	write_snake_segment(colour,x+4,y+8);
	write_snake_segment(colour,x+8,y+8);
	write_snake_segment(colour,x+12,y+8);
	write_snake_segment(colour,x+16,y+8);
	
	write_snake_segment(colour,x+16,y+12);
	
	write_snake_segment(colour,x,y+16);
	write_snake_segment(colour,x+4,y+16);
	write_snake_segment(colour,x+8,y+16);
	write_snake_segment(colour,x+12,y+16);
	write_snake_segment(colour,x+16,y+16);
}


void P(unsigned int x, unsigned int y, unsigned int colour)
{
	write_snake_segment(colour,x,y);
	write_snake_segment(colour,x+4,y);
	write_snake_segment(colour,x+8,y);
	write_snake_segment(colour,x+12,y);
	write_snake_segment(colour,x+16,y);
	
	write_snake_segment(colour,x,y+4);
	write_snake_segment(colour,x+16,y+4);
	
	write_snake_segment(colour,x,y+8);
	write_snake_segment(colour,x+4,y+8);
	write_snake_segment(colour,x+8,y+8);
	write_snake_segment(colour,x+12,y+8);
	write_snake_segment(colour,x+16,y+8);
	
	write_snake_segment(colour,x,y+12);
	
	write_snake_segment(colour,x,y+16);
}

void W(unsigned int x, unsigned int y, unsigned int colour)
{
	write_snake_segment(colour,x,y);
	write_snake_segment(colour,x+16,y);
	
	write_snake_segment(colour,x,y+4);
	write_snake_segment(colour,x+16,y+4);
	
	write_snake_segment(colour,x,y+8);
	write_snake_segment(colour,x+8,y+8);
	write_snake_segment(colour,x+16,y+8);
	
	write_snake_segment(colour,x,y+12);
	write_snake_segment(colour,x+8,y+12);
	write_snake_segment(colour,x+16,y+12);
	
	write_snake_segment(colour,x,y+16);
	write_snake_segment(colour,x+4,y+16);
	write_snake_segment(colour,x+8,y+16);
	write_snake_segment(colour,x+12,y+16);
	write_snake_segment(colour,x+16,y+16);
}

void I(unsigned int x, unsigned int y, unsigned int colour)
{
	write_snake_segment(colour,x,y);
	write_snake_segment(colour,x+4,y);
	write_snake_segment(colour,x+8,y);
	write_snake_segment(colour,x+12,y);
	write_snake_segment(colour,x+16,y);
	
	write_snake_segment(colour,x+8,y+4);
	
	write_snake_segment(colour,x+8,y+8);
	
	write_snake_segment(colour,x+8,y+12);
	
	write_snake_segment(colour,x,y+16);
	write_snake_segment(colour,x+4,y+16);
	write_snake_segment(colour,x+8,y+16);
	write_snake_segment(colour,x+12,y+16);
	write_snake_segment(colour,x+16,y+16);
}

void N(unsigned int x, unsigned int y, unsigned int colour)
{
	write_snake_segment(colour,x,y);
	write_snake_segment(colour,x+16,y);
	
	write_snake_segment(colour,x+2,y+2);
	
	write_snake_segment(colour,x,y+4);
	write_snake_segment(colour,x+4,y+4);
	write_snake_segment(colour,x+16,y+4);
	
	write_snake_segment(colour,x+6,y+6);
	
	write_snake_segment(colour,x,y+8);
	write_snake_segment(colour,x+8,y+8);
	write_snake_segment(colour,x+16,y+8);
	
	write_snake_segment(colour,x+10,y+10);
	
	write_snake_segment(colour,x,y+12);
	write_snake_segment(colour,x+12,y+12);
	write_snake_segment(colour,x+16,y+12);
	
	write_snake_segment(colour,x+14,y+14);
	
	write_snake_segment(colour,x,y+16);
	write_snake_segment(colour,x+16,y+16);
}

void T(unsigned int x, unsigned int y, unsigned int colour)
{
	write_snake_segment(colour,x,y);
	write_snake_segment(colour,x+4,y);
	write_snake_segment(colour,x+8,y);
	write_snake_segment(colour,x+12,y);
	write_snake_segment(colour,x+16,y);

	write_snake_segment(colour,x+8,y+4);
	
	write_snake_segment(colour,x+8,y+8);
	
	write_snake_segment(colour,x+8,y+12);
	
	write_snake_segment(colour,x+8,y+16);
}

void E(unsigned int x, unsigned int y, unsigned int colour)
{
	write_snake_segment(colour,x,y);
	write_snake_segment(colour,x+4,y);
	write_snake_segment(colour,x+8,y);
	write_snake_segment(colour,x+12,y);
	write_snake_segment(colour,x+16,y);
	
	write_snake_segment(colour,x,y+4);
	
	write_snake_segment(colour,x,y+8);
	write_snake_segment(colour,x+4,y+8);
	write_snake_segment(colour,x+8,y+8);
	
	write_snake_segment(colour,x,y+12);
	
	write_snake_segment(colour,x,y+16);
	write_snake_segment(colour,x+4,y+16);
	write_snake_segment(colour,x+8,y+16);
	write_snake_segment(colour,x+12,y+16);
	write_snake_segment(colour,x+16,y+16);
}

void A(unsigned int x, unsigned int y, unsigned int colour)
{
	write_snake_segment(colour,x,y);
	write_snake_segment(colour,x+4,y);
	write_snake_segment(colour,x+8,y);
	write_snake_segment(colour,x+12,y);
	write_snake_segment(colour,x+16,y);
	
	write_snake_segment(colour,x,y+4);
	write_snake_segment(colour,x+16,y+4);
	
	write_snake_segment(colour,x,y+8);
	write_snake_segment(colour,x+4,y+8);
	write_snake_segment(colour,x+8,y+8);
	write_snake_segment(colour,x+12,y+8);
	write_snake_segment(colour,x+16,y+8);
	
	write_snake_segment(colour,x,y+12);
	write_snake_segment(colour,x+16,y+12);

	write_snake_segment(colour,x,y+16);
	write_snake_segment(colour,x+16,y+16);
}

void Q(unsigned int x, unsigned int y, unsigned int colour)
{
	write_snake_segment(colour,x,y);
	write_snake_segment(colour,x+4,y);
	write_snake_segment(colour,x+8,y);
	write_snake_segment(colour,x+12,y);
	
	write_snake_segment(colour,x,y+4);
	write_snake_segment(colour,x+12,y+4);
	
	write_snake_segment(colour,x,y+8);
	write_snake_segment(colour,x+12,y+8);
	
	write_snake_segment(colour,x,y+12);
	write_snake_segment(colour,x+12,y+12);

	write_snake_segment(colour,x,y+16);
	write_snake_segment(colour,x+4,y+16);
	write_snake_segment(colour,x+8,y+16);
	write_snake_segment(colour,x+12,y+16);
	write_snake_segment(colour,x+16,y+16);
}

void R(unsigned int x, unsigned int y, unsigned int colour)
{
	write_snake_segment(colour,x,y);
	write_snake_segment(colour,x+4,y);
	write_snake_segment(colour,x+8,y);
	write_snake_segment(colour,x+12,y);
	write_snake_segment(colour,x+16,y);
	
	write_snake_segment(colour,x,y+4);
	write_snake_segment(colour,x+16,y+4);
	
	write_snake_segment(colour,x,y+8);
	write_snake_segment(colour,x+4,y+8);
	write_snake_segment(colour,x+8,y+8);
	write_snake_segment(colour,x+12,y+8);
	write_snake_segment(colour,x+16,y+8);
	
	write_snake_segment(colour,x+10,y+10);
	
	write_snake_segment(colour,x,y+12);
	write_snake_segment(colour,x+12,y+12);

	write_snake_segment(colour,x+14,y+14);
	
	write_snake_segment(colour,x,y+16);
	write_snake_segment(colour,x+16,y+16);
}

void apL(unsigned int x, unsigned int y, unsigned int colour)
{
	write_snake_segment(colour,x+12,y);
	
	write_snake_segment(colour,x+12,y+4);
}

void apR(unsigned int x, unsigned int y, unsigned int colour)
{
	write_snake_segment(colour,x+4,y);
	
	write_snake_segment(colour,x+4,y+4);
}


void update_score(unsigned int score, unsigned int colour, unsigned int player){
		
		unsigned int score_x;
		unsigned int score_y;
		unsigned int gem;
		
		if( player == 1 ){
			score_x = 41;
			score_y = 50;
			gem = 18;
		}
		else{
			score_x = 601;
			score_y = 50;
			gem = 578;
		}

		switch(score) {
			case(0):
				zero(score_x, score_y, colour);
				break;
			case(1):
				one(score_x, score_y, colour);
				add_big_gem(gem,480-score*40);
				break;
			case(2):
				two(score_x, score_y, colour);
				add_big_gem(gem,480-score*40);
				break;
			case(3):
				three(score_x, score_y, colour);
				add_big_gem(gem,480-score*40);
				break;
			case(4):
				four(score_x, score_y, colour);
				add_big_gem(gem,480-score*40);
				break;
			case(5):
				five(score_x, score_y, colour);
				add_big_gem(gem,480-score*40);
				break;
			case(6):
				six(score_x, score_y, colour);
				add_big_gem(gem,480-score*40);
				break;
			case(7):
				seven(score_x, score_y, colour);
				add_big_gem(gem,480-score*40);
				break;
			case(8):
				eight(score_x, score_y, colour);
				add_big_gem(gem,480-score*40);
				break;
			case(9):
				nine(score_x, score_y, colour);
				add_big_gem(gem,480-score*40);
				break;
			case(10):
				one(score_x-22, score_y, colour);
				zero(score_x, score_y, colour);
				add_big_gem(gem,480-score*40);
				break;
		}
}

void player_labels(){

	int i;

	//player 1
	P(19, 10, SNAKE1_COLOUR);
	one( 41, 10, SNAKE1_COLOUR );
	
	for( i=0; i<11; i++ ){
		write_snake_segment(SNAKE1_COLOUR,(18+i*4),32);
	}

	//player2
	P(579, 10, SNAKE2_COLOUR);
	two( 601, 10, SNAKE2_COLOUR );
	
	for( i=0; i<11; i++ ){
		write_snake_segment(SNAKE2_COLOUR,(578+i*4),32);
	}

}

void start_label( unsigned int colour ){

	P(233,10,colour);
	R(255,10,colour);
	E(277,10,colour);
	five(299,10,colour);
	five(321,10,colour);
	
	apL(354,10,colour);
	Q(376,10,colour);
	apR(398,10,colour);
	
	T(233,50,colour);
	zero(255,50,colour);
	
	five(299,50,colour);
	T(321,50,colour);
	A(343,50,colour);
	R(365,50,colour);
	T(387,50,colour);
}

/////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////// SNAKE //////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////


//initialize snake struct
struct Snake* buildSnake( struct Snake* snake, unsigned int player ){

	snake = (struct Snake*)malloc(sizeof(struct Snake));

	snake->player_number = player;
	snake->length = 15;
	
	if( player == 1 ){
		snake->head_address = SNAKE1_BASEADDR;
	}
	else if( player == 2 ){
		snake->head_address = SNAKE2_BASEADDR;
	}

	return snake;
}

//reset position of the snake
void reset_snake ( struct Snake* snake ){

	int x;
	int y;
	int i;
	int next;
	
	if(snake->player_number == 1){
		x = SNAKE1_START_X_POS;
		y = SNAKE1_START_Y_POS;
		next = -SEGMENT_SIZE;
	}
	else if(snake->player_number == 2){
		x = SNAKE2_START_X_POS;
		y = SNAKE2_START_Y_POS;
		next = SEGMENT_SIZE;
	}

	snake->length = 15;

	//head     tail
	//[0][1]...[28][29]
	for( i=0; i<snake->length; i++){

		( (int *) snake->head_address)[i*2] = x;
		( (int *) snake->head_address)[i*2+1] = y;

		x += next;
	}
}

//draws entire snake (usually for starting/reset)
void draw_snake(struct Snake* snake){

	int x;
	int y;
	int i;
	unsigned int colour;

	if( snake->player_number == 1 ){
		colour = SNAKE1_COLOUR;
	}
	else{
		colour = SNAKE2_COLOUR;
	}
	
	for( i=0; i<snake->length; i++){
		x = ( (int *) snake->head_address)[i*2];
		y = ( (int *) snake->head_address)[i*2+1];
	
		write_snake_segment( colour, x, y );
	}
}

//moves snake forward one segment
void update_snake( struct Snake * snake, int x, int y ){

	int i;
	
	unsigned int colour;
	
	if( snake->player_number == 1 ){
		colour = SNAKE1_COLOUR;
	}
	else{
		colour = SNAKE2_COLOUR;
	}
	
	//erase tail segment from video memory
	write_snake_segment( (BACKGROUND_COLOUR), ( (int *) snake->head_address)[(snake->length-1)*2], ( (int *) snake->head_address)[(snake->length-1)*2+1] );

	//shift x and y coordinates from head -> tail (ie. tail gets [x,y] from segment before it, etc)
	for( i=snake->length; i>0; i-- ){
		( (int *) snake->head_address)[i*2] = ( (int *) snake->head_address)[(i-1)*2];
		( (int *) snake->head_address)[i*2+1] = ( (int *) snake->head_address)[(i-1)*2+1];
	}
	
	//give head segment new coordinates
	( (int *) snake->head_address)[0] = x;
	( (int *) snake->head_address)[1] = y;

	//write it to video memory
	write_snake_segment( colour, x, y );

}

//grow snake by one segment until it reaches max length
void grow_snake ( struct Snake * snake ){
		
	if( snake->length < MAX_SNAKE_LENGTH ){
		int i = snake->length;

		( (int *) snake->head_address)[i*2] = ( (int *) snake->head_address)[(i-1)*2]-2;
		( (int *) snake->head_address)[i*2+1] = ( (int *) snake->head_address)[(i-1)*2+1];
		
		( (int *) snake->head_address)[(i+1)*2] = ( (int *) snake->head_address)[(i)*2]-2;
		( (int *) snake->head_address)[(i+1)*2+1] = ( (int *) snake->head_address)[(i)*2+1];

		( (int *) snake->head_address)[(i+2)*2] = ( (int *) snake->head_address)[(i+1)*2]-2;
		( (int *) snake->head_address)[(i+2)*2+1] = ( (int *) snake->head_address)[(i+1)*2+1];

		snake->length += 2;
	}
}


/////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////// GAME ///////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////


//player 1 or 2 (so writes to [0] or [1])
void write_score(unsigned int value, unsigned int player)
{
	( (unsigned int *) SCORE_BASEADDR)[player-1] = value;
}

//player 1 or 2 (so reads from [0] or [1])
unsigned int read_score( unsigned int player )
{
	return ( (unsigned int *) SCORE_BASEADDR)[player-1];
}
